// generated by Fast Light User Interface Designer (fluid) version 1.00

#ifndef xfhdlcst_h
#define xfhdlcst_h
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
extern Fl_Window *hdlcst;
#include <FL/Fl_Box.H>
#include <FL/Fl_Round_Button.H>
extern Fl_Round_Button *dcd;
extern Fl_Round_Button *ptt;
#include <FL/Fl_Output.H>
extern Fl_Output *txpacket;
extern Fl_Output *txerror;
extern Fl_Output *rxpacket;
extern Fl_Output *rxerror;
extern Fl_Output *modcyc;
extern Fl_Output *demodcyc;
extern Fl_Output *intfreq;
extern Fl_Output *dmares;
extern Fl_Box *tmodcyc;
extern Fl_Box *tdemodcyc;
extern Fl_Box *tintfreq;
extern Fl_Box *tdmares;
extern Fl_Output *modename;
extern Fl_Output *drivername;
#include <FL/Fl_Button.H>
extern void cb_quit(Fl_Button*, long);
extern Fl_Button *quit;
Fl_Window* create_the_forms();
#endif
